Package: fenr
Title: Fast functional enrichment for interactive applications
Version: 1.10.1
Authors@R: person(
    given = "Marek",
    family = "Gierlinski",
    email = "M.Gierlinski@dundee.ac.uk",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-9149-3514")
    ) 
Description: Perform fast functional enrichment on feature lists (like genes
    or proteins) using the hypergeometric distribution. Tailored for speed,
    this package is ideal for interactive platforms such as Shiny. It supports
    the retrieval of functional data from sources like GO, KEGG, Reactome,
    Bioplanet and WikiPathways. By downloading and preparing data first, it
    allows for rapid successive tests on various feature selections without
    the need for repetitive, time-consuming preparatory steps typical of other
    packages. 
URL: https://github.com/bartongroup/fenr
BugReports: https://github.com/bartongroup/fenr/issues
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: FunctionalPrediction, DifferentialExpression,
        GeneSetEnrichment, GO, KEGG, Reactome, Proteomics
Imports: tools, methods, assertthat, rlang, dplyr, tidyr, tidyselect,
        tibble, purrr, readr, stringr, httr2, rvest, progress,
        BiocFileCache, shiny, ggplot2
Suggests: BiocStyle, testthat, knitr, rmarkdown, topGO
VignetteBuilder: knitr
LazyData: false
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/fenr
git_branch: RELEASE_3_23
git_last_commit: 6578bfb
git_last_commit_date: 2026-06-30
Repository: Bioconductor 3.23
Date/Publication: 2026-07-02
NeedsCompilation: no
Packaged: 2026-07-02 21:48:17 UTC; biocbuild
Author: Marek Gierlinski [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9149-3514>)
Maintainer: Marek Gierlinski <M.Gierlinski@dundee.ac.uk>
