# Generated by roxygen2: do not edit by hand

export(enrichment_interactive)
export(fetch_bp)
export(fetch_go)
export(fetch_go_species)
export(fetch_kegg)
export(fetch_kegg_species)
export(fetch_reactome)
export(fetch_reactome_species)
export(fetch_terms_for_example)
export(fetch_wiki)
export(fetch_wiki_species)
export(functional_enrichment)
export(get_feature_terms)
export(get_term_features)
export(prepare_for_enrichment)
export(remove_cache)
importFrom(assertthat,assert_that)
importFrom(assertthat,is.count)
importFrom(assertthat,is.string)
importFrom(methods,is)
