#' @description
#'
#' This R package provides a fast and efficient method for functional enrichment
#' analysis, optimized for speed and designed for use in interactive
#' applications, such as *Shiny* apps.
#'
#' To learn more about fenr, start with the vignette: vignette("fenr").
#'
#' @keywords internal
#' @return Package fenr.
"_PACKAGE"
