% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exmpl_all}
\alias{exmpl_all}
\title{Example set of background genes.}
\format{
A character vector with 6985 elements.
}
\usage{
data(exmpl_all)
}
\value{
A vector with background gene names.
}
\description{
A set of gene names for proteins that could be detected in a typical
proteomics experiment on yeast samples.
}
\keyword{datasets}
